#ifndef _M3DMATH_H_
#define _M3DMATH_H_

#define PI 3.14159265358979323846
#define DEG2RAD (PI/180.0)
#define RAD2DEG (180.0/PI)

// trieda pre pracu s 3D bodmi a vektormi
class m3Dvector {
public:
 // konstruktor
 m3Dvector() {}
 // konstruktor s inicializaciou dat
 m3Dvector(float a,float b,float c) { x=a; y=b; z=c; }

 // pretazeny operator +
 m3Dvector operator+(m3Dvector mv) { return m3Dvector(mv.x+x,mv.y+y,mv.z+z); }
 // pretazeny operator -
 m3Dvector operator-(m3Dvector mv) { return m3Dvector(x-mv.x,y-mv.y,z-mv.z); }
 // pretazeny operator *
 m3Dvector operator*(float num) { return m3Dvector(x*num,y*num,z*num); }
 // hodnoty vektora (poloha 3D bodu)
 float x,y,z;

 // velkost vektora
 float M;
 // vypocet velkosti vektora
 float Magnitude(void);
 // normalizacia vektora
 void Normalize(void);
};

// trieda matematicko-geometrickych vypoctov
class m3Dmath {
public:
 // konstruktor
 m3Dmath() {}

 // test kolizie kamery s trojuholnikom
 bool CameraInTriangle(m3Dvector triangle[],m3Dvector trianglenormal,m3Dvector line[],
                       m3Dvector triangleplanes[],float D1,float D2, float D3);

 // test kolizie kamery s trojuholnikom - s vyuzitim gule
 bool CameraInTriangleS(m3Dvector triangle[],m3Dvector trianglenormal,m3Dvector center,float radius,
                        m3Dvector triangleplanes[],float D1,float D2, float D3);

 // nova poloha kamery ovplyvnena koliziou s objektom sceny
 m3Dvector ret_pos;
private:
 int i; // pomocna premenna

 float distance0,distance1,distance2; // pre vypocet vzdialenosti
 m3Dvector m_vector,m_point;          // pre docasne ulozenie vektora a bodu
 m3Dvector m_vectA,m_vectB;
 float Numerator,Denominator,D,M;     // pre vypocet priesecnika bodu s rovinou

 // vypocet vzdialenosti trojuholnika od pociatku suradnicovej sustavy
 float TriangleDistance(m3Dvector normal,m3Dvector point);

 // test kolizie pohyboveho vektora kamery s trojuholnikom
 bool IntersectedTriangle(m3Dvector triangle[],
                          m3Dvector trianglenormal,
                          m3Dvector line[],
                          float *dist);

 // normalizacia vektora
 m3Dvector Normalize(m3Dvector vector);

 // vektorovy sucin dvoch vektorov
 void Cross(m3Dvector vector1,m3Dvector vector2,m3Dvector vector3);
 // skalarny sucin dvoch vektorov
 float Dot(m3Dvector vector1,m3Dvector vector2);

 // vzdialenost dvoch bodov
 float Distance(m3Dvector pA,m3Dvector pB);

 // vypocet priesecnika roviny, ktoru tvori trojuholnik a pohyboveho vektora kamery
 m3Dvector IntersectionPoint(m3Dvector trianglenormal,m3Dvector line[],float dist);

 // test ci priesecnik patri trojuholniku, ak nie, nedoslo ku kolizii
 bool PointInTriangle(m3Dvector IPoint,m3Dvector triangleplanes[],float D1,float D2,float D3);

 // vrati absolutnu hodnotu cisla
 float myAbs(float N);

 // test kolizie gule s trojuholnikom
 bool IntersectedSphere(m3Dvector triangle[],
                        m3Dvector trianglenormal,
                        m3Dvector center,float radius,
                        float *dist);

 // test kolizie gule s hranami trojuholnika
 bool EdgeCollision(m3Dvector center,m3Dvector triangle[],float radius);

 // vrati bod na ciare pA-pB, ktory je najblizsie k bodu pC
 m3Dvector PointOnLine(m3Dvector pA,m3Dvector pB,m3Dvector pC);
};

/*
  pri tvorbe tohto kodu som sa inspiroval zverejnenym kodom autora:

  Ben Humphrey (DigiBen)
  Game Programmer
  DigiBen@GameTutorials.com
  Co-Web Host of www.GameTutorials.com
  date: 1/23/02
*/

#endif
