// hlavickove subory *******************************************************
// systemove funkcie
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
// GL a GLU kniznica
#include <GL/gl.h>
//#include <GL/glu.h>

#include "mFire.h"

// konstruktor *************************************************************
mFire::mFire()
{
 p_system=NULL;

 sp=0.0f; ti=0.0f;
}

// destruktor **************************************************************
mFire::~mFire()
{
 if (p_system!=NULL) {
  if (p_system->xyz!=NULL) { free(p_system->xyz); p_system->xyz=NULL; }	 
  if (p_system->speed!=NULL) { free(p_system->speed); p_system->speed=NULL; }	 
  if (p_system->time!=NULL) { free(p_system->time); p_system->time=NULL; }	 
  free(p_system); p_system=NULL;  
 }
}

// nahodny pohyb castic systemu ********************************************
float mFire::noise(float a)
{
 return sqrt(-2.0f*log((float) rand()/RAND_MAX))*sin(2.0f*PI*(float) rand()/RAND_MAX)*a;
}

// vytvorenie systemu ******************************************************
int mFire::create(GLuint N,GLfloat px,GLfloat py,GLfloat pz,
                  GLfloat speed,GLfloat time,GLfloat r,
                  GLuint tid)
{
 sp=speed; ti=time;

 // miesto pre strukturu particle_system
 p_system=(particle_system *) malloc(sizeof(particle_system));
 if (p_system==NULL) {
  fprintf(stderr,"Chyba alokacie pamate pre particle_system!\n"); return(1);	 
 }
 
 // miesto pre udaje jednotlivych castic
 p_system->N=N;
 p_system->xyz=(GLfloat *) malloc(3*N*sizeof(GLfloat));
 if (p_system->xyz==NULL) {
  fprintf(stderr,"Chyba alokacie pamate pre particle_system!\n"); return(1);	 
 }
 p_system->speed=(GLfloat *) malloc(3*N*sizeof(GLfloat));
 if (p_system->speed==NULL) {
  fprintf(stderr,"Chyba alokacie pamate pre particle_system!\n"); return(1);	 
 }
 p_system->time=(GLfloat *) malloc(N*sizeof(GLfloat));
 if (p_system->time==NULL) {
  fprintf(stderr,"Chyba alokacie pamate pre particle_system!\n"); return(1);	 
 }

 // naplnenie struktury particle_system
 p_system->s_pos.x=px; p_system->s_pos.y=py; p_system->s_pos.z=pz;
 p_system->s_time=time;
 p_system->s_speed=speed;
 p_system->s_radius=r;

 for (ta=0;ta<N;ta++) {
  p_system->xyz[ta*3+0]=px;
  p_system->xyz[ta*3+1]=py;
  p_system->xyz[ta*3+2]=pz;

  p_system->speed[ta*3+0]=noise(speed);
  p_system->speed[ta*3+1]=noise(speed);
  p_system->speed[ta*3+2]=noise(speed);

  p_system->time[ta]=(GLfloat) ta/(GLfloat) N*time;
 }

 p_system->texture_id=tid;

 p_system->color.x=1.0f; p_system->color.y=1.0f; p_system->color.z=1.0f;

 return(0);
}

// nastavenie farby systemu ************************************************
void mFire::setcolor(GLfloat c1,GLfloat c2,GLfloat c3,GLfloat c4)
{
 p_system->color.x=c1; p_system->color.y=c2; p_system->color.z=c3;
}

// "zivot systemu" *********************************************************
void mFire::update(GLfloat f1,GLfloat f2,GLfloat f3,GLfloat tadd)
{
 // posun castic systemu v smere posobenia sily
 f.x=f1*tadd; f.y=f2*tadd; f.z=f3*tadd;

 for (ta=0,xyz=p_system->xyz,speed=p_system->speed;ta<p_system->N;ta++,xyz+=3,speed+=3) {
  // zvysenie rychlosti castice
  speed[0]+=f.x; speed[1]+=f.y; speed[2]+=f.z;
  // casova korekcia
  s.x=speed[0]*tadd; s.y=speed[1]*tadd; s.z=speed[2]*tadd;
  // zmena polohy castice 
  xyz[0]+=s.x; xyz[1]+=s.y; xyz[2]+=s.z;
  // zvysenie casu "zivota" castice
  p_system->time[ta]+=tadd;
  // po prekroceni stanoveneho casu sa castica resetuje
  if (p_system->time[ta]>p_system->s_time) {
   xyz[0]=p_system->s_pos.x;
   xyz[1]=p_system->s_pos.y;
   xyz[2]=p_system->s_pos.z;
   
   speed[0]=noise(p_system->s_speed);
   speed[1]=noise(p_system->s_speed);
   speed[2]=noise(p_system->s_speed);

   p_system->time[ta]=0.0f;
  }
 }
}

// vykreslenie systemu *****************************************************
void mFire::render(void)
{
 // ulozenie potrebnych atributov
 glPushAttrib(GL_DEPTH_BUFFER_BIT|GL_TEXTURE_BIT|GL_LIGHTING_BIT);
 glDisable(GL_DEPTH_TEST); // vypnutie testu hlbky
 glDisable(GL_LIGHTING);   // vypnutie osvetlenia
 glEnable(GL_TEXTURE_2D);  // zapnutie texturovania

 glBlendFunc(GL_ONE,GL_ONE);
 glEnable(GL_BLEND);
 
 // ziskanie modelview matice
 glGetFloatv(GL_MODELVIEW_MATRIX,mvmatrix);
 // vypocet prirastkov suradnic castic tak, aby tieto boli
 // vzdy natocene kolmo na pohlad kamery
 // rozmer x - sirka castice
 mx.x=1.0f; mx.y=0.0f; mx.z=0.0f;
 s.x=(mx.x*mvmatrix[0]+mx.y*mvmatrix[1]+mx.z*mvmatrix[2])*p_system->s_radius;
 s.y=(mx.x*mvmatrix[4]+mx.y*mvmatrix[5]+mx.z*mvmatrix[6])*p_system->s_radius;
 s.z=(mx.x*mvmatrix[8]+mx.y*mvmatrix[9]+mx.z*mvmatrix[10])*p_system->s_radius;
 mx.x=s.x; mx.y=s.y; mx.z=s.z;
 // rozmer y - vyska castice
 my.x=0.0f; my.y=1.0f; my.z=0.0f;
 s.x=(my.x*mvmatrix[0]+my.y*mvmatrix[1]+my.z*mvmatrix[2])*p_system->s_radius;
 s.y=(my.x*mvmatrix[4]+my.y*mvmatrix[5]+my.z*mvmatrix[6])*p_system->s_radius;
 s.z=(my.x*mvmatrix[8]+my.y*mvmatrix[9]+my.z*mvmatrix[10])*p_system->s_radius;
 my.x=s.x; my.y=s.y; my.z=s.z;
 
 // zvolenie patricnej textury
 glBindTexture(GL_TEXTURE_2D,p_system->texture_id);
 // vykreslenie vsetkych castic
 for(ta=0,xyz=p_system->xyz;ta<p_system->N;ta++,xyz+=3) {
  // upravenie farby cestice, po polovici "zivota" je castica uz iba "dym"
  if (p_system->time[ta]>p_system->s_time/2.0f) {
   p_color[0]=(0.75f-p_system->time[ta]/p_system->s_time);
   p_color[1]=(0.75f-p_system->time[ta]/p_system->s_time);
   p_color[2]=(0.75f-p_system->time[ta]/p_system->s_time);
  }
  else {
   p_color[0]=p_system->color.x*(1.0f-p_system->time[ta]/p_system->s_time);
   p_color[1]=p_system->color.y*(1.0f-p_system->time[ta]/p_system->s_time);
   p_color[2]=p_system->color.z*(1.0f-p_system->time[ta]/p_system->s_time);
  }
  p_color[3]=1.0f;
  glColor4fv(p_color);
  // castica sa kresli ako pas trojuholnikov - vytvori stvorec
  glBegin(GL_TRIANGLE_STRIP);
   glTexCoord2f(0.0f,1.0f);
   glVertex3f(xyz[0]-mx.x-my.x,xyz[1]-mx.y-my.y,xyz[2]-mx.z-my.z);
   glTexCoord2f(1.0f,1.0f);
   glVertex3f(xyz[0]+mx.x-my.x,xyz[1]+mx.y-my.y,xyz[2]+mx.z-my.z);
   glTexCoord2f(0.0f,0.0f);
   glVertex3f(xyz[0]-mx.x+my.x,xyz[1]-mx.y+my.y,xyz[2]-mx.z+my.z);
   glTexCoord2f(1.0f,0.0f);
   glVertex3f(xyz[0]+mx.x+my.x,xyz[1]+mx.y+my.y,xyz[2]+mx.z+my.z);
  glEnd();
 }
 glColor4f(1.0f,1.0f,1.0f,1.0f); // len pre korekciu
 
 glDisable(GL_BLEND);
 
 glPopAttrib(); // vyber ulozenych atributov
}

void mFire::resettime(void)
{
 for (ta=0;ta<p_system->N;ta++) {
  p_system->speed[ta*3+0]=noise(sp);
  p_system->speed[ta*3+1]=noise(sp);
  p_system->speed[ta*3+2]=noise(sp);

  p_system->time[ta]=(GLfloat) ta/(GLfloat) p_system->N*ti;
 }
}

/*
  pri priprave tohto kodu som sa inspiroval zverejnenym kodom autora:

  Alexander Zaprjagaev
  frustum@frustum.tomsk.ru
  http://frustum.tomsk.ru
*/
