#ifndef _MOGL_H_
#define _MOGL_H_

// vlastne triedy
#include "mCamera.h"
#include "mText.h"
#include "mTexture.h"
#include "mObjects.h"
#include "mExt.h"
#include "m3Dmath.h"
#include "mFire.h"

typedef struct __material { // struktura OpenGL materialu
 GLfloat amb[4],dif[4],spe[4],amb_dif[4],emi[4];
 GLfloat shi,transp,sillum[4];
} _material;

typedef struct __light {    // struktura OpenGL svetla
 GLfloat amb[4],dif[4],spe[4];
 GLfloat pos[4],spot[3];
} _light;

// trieda premennych a funkcii pre pracu s OpenGL
class mOGL {
public:
 mOGL();          // konstruktor
 virtual ~mOGL(); // destruktor

 mCamera m_Camera[8]; // pouzite kamery

 bool gl_initialize(void); // inicializacia OpenGL
 int gl_draw(void);        // vykreslenie OpenGL sceny
 int gl_iddle(void);       // casovo zavisla uprava objektov

 // zmena stavov OpenGL stavoveho stroja -----------------------------------
 void change_PolygonMode(void);   // zmena sposobu zobrazenia objektov
 void change_LightingOnOff(void); // zap/vyp osvetlenia
 void change_NormalsOnOff(void);  // zap/vyp vykreslovania normal
 void change_FogOnOff(void);      // zap/vyp hmly
 void change_FogMode(void);       // zmena parametrov hmly
 void change_TexOnOff(void);      // zap/vyp texturovania
 void change_HelpOnOff(void);     // zap/vyp vypisu helpu
 void change_PauseMotion(void);   // zap/vyp casovo zavisleho pohybu objektov

 // pouzite triedy ---------------------------------------------------------
 mFire ogl_Fire; // instancia triedy mFire - ohen
private:
 GLfloat StartInfoPos;                // pozicia vypisu start info
 void PrintStartInfo(char *infotext); // vypis info pocas startu aplikacie

 _material ogl_Material[8]; // pouzite materialy
 _light ogl_Light[8];       // pouzite svetla
 GLfloat ogl_FogColor[4];   // farba hmly

 GLuint t_a,t_b; // pomocne premenne

 GLint ogl_State[2]; // pre zistovanie stavov OpenGL stavoveho stroja

 GLubyte normals_onoff;  // indikator zap/vyp vykreslovania normal
 GLubyte help_onoff;     // indikator zap/vyp vypisu helpu
 GLubyte lighting_onoff; // indikator zap/vyp osvetlenia
 GLubyte fog_onoff;      // indikator zap/vyp hmly
 GLubyte pause_motion;   // indikator zap/vyp casovo zavisleho pohybu objektov

 char fname[128];      // pre ulozenie retazcov - cesta k suboru

 GLfloat rot_x,rot_y; // premenne pre aplikaciu rotacie objektov sceny
 GLfloat pos_y,add_y; // premenne pre aplikaciu posunu objektov sceny

 // rovnica orezavacej roviny
 double plane_equ[4];

 // pouzite triedy ---------------------------------------------------------
 mText ogl_Text;             // instancia triedy mText - praca s textom
 mTexture ogl_Texture;       // instancia triedy mTexture - praca s texturou
 mObjects ogl_Objects;       // instancia triedy mObjects - jednoduche objekty
 //mExt ogl_Ext;               // instancia triedy mExt - OpenGL rozsirenia
 //m3Dmath ogl_3Dmath;         // instancia triedy m3Dmath - vypocty
};

#endif
